@@ -40,6 +40,14 @@
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#if defined(AF_LINK) /* BSD */
+#if defined(__OpenBSD__) || defined(__NetBSD__)
+#include <sys/ioctl.h>
+#include <net/if_media.h>
+#endif
+#include <net/if.h>
+#include <net/if_types.h>
+#endif
 #include <ifaddrs.h>
 
 #define BORDER 8
@@ -58,7 +66,6 @@ typedef struct
   gboolean show_bar;
   gchar *command;
 
-  int size;
   int signal_strength;
   GtkOrientation orientation;
 
@@ -67,9 +74,7 @@ typedef struct
   GtkWidget *image;
   GtkWidget *signal;
   GtkWidget *tooltip_text;
-#if GTK_CHECK_VERSION (3, 16, 0)
   GtkCssProvider *css_provider;
-#endif
 
   XfcePanelPlugin *plugin;
 
@@ -160,14 +165,10 @@ wavelan_update_signal(t_wavelan *wavelan)
   gchar signal_color_weak[] = "#e05200";
   gchar signal_color_good[] = "#e6ff00";
   gchar signal_color_strong[] = "#06c500";
-#if GTK_CHECK_VERSION (3, 16, 0)
   gchar *css, *color_str;
-#if GTK_CHECK_VERSION (3, 20, 0)
   gchar * cssminsizes = "min-width: 4px; min-height: 0px";
   if(gtk_orientable_get_orientation(GTK_ORIENTABLE(wavelan->signal)) == GTK_ORIENTATION_HORIZONTAL)
     cssminsizes = "min-width: 0px; min-height: 4px";
-#endif
-#endif
 
   if (!wavelan->show_bar) {
     gtk_widget_hide(wavelan->signal);
@@ -190,41 +191,21 @@ wavelan_update_signal(t_wavelan *wavelan)
    else
     gdk_rgba_parse(&color, signal_color_bad);
 
-#if GTK_CHECK_VERSION (3, 16, 0)
      color_str = gdk_rgba_to_string(&color);
-#if GTK_CHECK_VERSION (3, 20, 0)
      css = g_strdup_printf("progressbar trough { %s } \
                             progressbar progress { %s ; background-color: %s; background-image: none; }",
                            cssminsizes, cssminsizes,
-#else
-     css = g_strdup_printf(".progressbar { background-color: %s; background-image: none; }",
-#endif
                            color_str);
      g_free(color_str);
-#else
-     gtk_widget_override_background_color(GTK_WIDGET(wavelan->signal),
-                             GTK_STATE_PRELIGHT,
-                             &color);
-     gtk_widget_override_background_color(GTK_WIDGET(wavelan->signal),
-                             GTK_STATE_SELECTED,
-                             &color);
-     gtk_widget_override_color(GTK_WIDGET(wavelan->signal),
-                             GTK_STATE_SELECTED,
-                             &color);
-#endif
   } else {
-#if GTK_CHECK_VERSION (3, 20, 0)
      /* only set size... */
      css = g_strdup_printf("progressbar trough { %s } \
                             progressbar progress { %s }",
                            cssminsizes, cssminsizes);
-#endif
   }
 
-#if GTK_CHECK_VERSION (3, 16, 0)
   gtk_css_provider_load_from_data (wavelan->css_provider, css, strlen(css), NULL);
   g_free(css);
-#endif
 
   gtk_widget_show(wavelan->signal);
 }
@@ -345,24 +326,44 @@ wavelan_query_interfaces (void)
 {
   GList *interfaces = NULL;
   struct ifaddrs *ifaddr, *ifa;
+#if defined(__OpenBSD__) || defined(__NetBSD__)
+  int sock;
+#endif
 
   TRACE ("Entered wavelan_query_interface");
 
   if (getifaddrs(&ifaddr) == -1) {
     return NULL;
   }
+#if defined(__OpenBSD__) || defined(__NetBSD__)
+  if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+    return NULL;
+#endif
   for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
     if (ifa->ifa_addr == NULL)
       continue;
 #if defined(AF_LINK) /* BSD */
-    if (ifa->ifa_addr->sa_family == AF_LINK)
+    if (ifa->ifa_addr->sa_family == AF_LINK &&
+        ((struct if_data *)ifa->ifa_data)->ifi_type == IFT_ETHER) {
+#if defined(__OpenBSD__) || defined(__NetBSD__)
+      struct ifmediareq ifmr;
+
+      bzero(&ifmr, sizeof(ifmr));
+      strlcpy(ifmr.ifm_name, ifa->ifa_name, sizeof(ifmr.ifm_name));
+      if (ioctl(sock, SIOCGIFMEDIA, &ifmr) < 0 || (ifmr.ifm_active & IFM_IEEE80211) == 0)
+        continue;
+#endif
 #elif defined(AF_PACKET) /* linux */
-    if (ifa->ifa_addr->sa_family == AF_PACKET)
+    if (ifa->ifa_addr->sa_family == AF_PACKET) {
 #else
 #error "couldnt find a way to get address family on your system"
 #endif
       interfaces = g_list_append (interfaces, g_strdup (ifa->ifa_name));
+    }
   }
+#if defined(__OpenBSD__) || defined(__NetBSD__)
+  close(sock);
+#endif
   freeifaddrs(ifaddr);
   return interfaces;
 }
@@ -428,9 +429,9 @@ wavelan_icon_clicked(GtkWidget *widget, gpointer data,t_wavelan *wavelan)
   if (wavelan->command == NULL || strlen(wavelan->command) == 0)
     return;
 
-  if (!xfce_spawn_command_line_on_screen (gtk_widget_get_screen (GTK_WIDGET (widget)),
-                                            wavelan->command,
-                                          FALSE, FALSE, &error))
+  if (!xfce_spawn_command_line (gtk_widget_get_screen (GTK_WIDGET (widget)),
+                                wavelan->command,
+                                FALSE, FALSE, TRUE, &error))
     {
       message_dialog = gtk_message_dialog_new_with_markup (NULL,
                                                            GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -462,7 +463,9 @@ wavelan_new(XfcePanelPlugin *plugin)
   wavelan->signal_colors = TRUE;
   wavelan->show_icon = TRUE;
   wavelan->show_bar = TRUE;
+#if defined(__linux__)
   wavelan->command = g_strdup("nm-connection-editor");
+#endif
   wavelan->state = -2;
 
   wavelan->plugin = plugin;
@@ -501,7 +504,7 @@ wavelan_new(XfcePanelPlugin *plugin)
   gtk_box_pack_start(GTK_BOX(wavelan->box), GTK_WIDGET(wavelan->signal), FALSE, FALSE, 0);
 
 
-  wavelan_set_size(plugin, xfce_panel_plugin_get_size (plugin), wavelan);
+  wavelan_set_size(plugin, xfce_panel_plugin_get_icon_size (plugin), wavelan);
   wavelan_set_orientation(plugin, xfce_panel_plugin_get_orientation (plugin),  wavelan);
   gtk_widget_show_all(wavelan->box);
   gtk_container_add(GTK_CONTAINER(wavelan->ebox), GTK_WIDGET(wavelan->box));
@@ -583,10 +586,6 @@ wavelan_set_orientation(XfcePanelPlugin* plugin, GtkOrientation orientation, t_w
   gtk_orientable_set_orientation(GTK_ORIENTABLE(wavelan->box), orientation);
   gtk_orientable_set_orientation(GTK_ORIENTABLE(wavelan->signal), !orientation);
   gtk_progress_bar_set_inverted(GTK_PROGRESS_BAR(wavelan->signal), (orientation == GTK_ORIENTATION_HORIZONTAL));
-  if (orientation == GTK_ORIENTATION_HORIZONTAL)
-   gtk_widget_set_size_request(wavelan->ebox, -1, wavelan->size);
-  else
-   gtk_widget_set_size_request(wavelan->ebox, wavelan->size, -1);
   wavelan_set_state(wavelan, wavelan->state);
 }
 
@@ -598,14 +597,9 @@ wavelan_set_size(XfcePanelPlugin* plugin, int size, t_wavelan *wavelan)
   size /= xfce_panel_plugin_get_nrows(plugin);
   xfce_panel_plugin_set_small (plugin, TRUE);
   border_width = size > 26 ? 2 : 1;
-  wavelan->size = size;
-  image_size = wavelan->size - (2 * border_width);
+  image_size = xfce_panel_plugin_get_icon_size (plugin);
   gtk_image_set_pixel_size (GTK_IMAGE (wavelan->image), image_size);
   gtk_container_set_border_width(GTK_CONTAINER(wavelan->box), border_width);
-  if (wavelan->orientation == GTK_ORIENTATION_HORIZONTAL)
-   gtk_widget_set_size_request(wavelan->ebox, -1, wavelan->size);
-  else
-   gtk_widget_set_size_request(wavelan->ebox, wavelan->size, -1);
 }
 
 /* interface changed callback */
@@ -693,11 +687,10 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 
   TRACE ("Entered wavelan_create_options");
   
-  dlg = xfce_titled_dialog_new_with_buttons (_("Wavelan Plugin Options"),
+  dlg = xfce_titled_dialog_new_with_mixed_buttons (_("Wavelan Plugin Options"),
               GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
               GTK_DIALOG_DESTROY_WITH_PARENT,
-              "gtk-close",
-              GTK_RESPONSE_OK,
+              "window-close-symbolic", _("_Close"), GTK_RESPONSE_OK,
               NULL);
 
   gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
@@ -828,21 +821,16 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 static void
 wavelan_show_about (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 {
-   GdkPixbuf *icon;
    const gchar *auth[] = { "Benedikt Meurer <benny at xfce.org>", "Florian Rivoal <frivoal@xfce.org>", NULL };
-   icon = xfce_panel_pixbuf_from_source("network-wireless", NULL, 32);
    gtk_show_about_dialog(NULL,
-      "logo", icon,
+      "logo-icon-name", "network-wireless",
       "license", xfce_get_license_text (XFCE_LICENSE_TEXT_BSD),
       "version", PACKAGE_VERSION,
       "program-name", PACKAGE_NAME,
       "comments", _("View the status of a wireless network"),
       "website", "https://docs.xfce.org/panel-plugins/xfce4-wavelan-plugin",
-      "copyright", "Copyright (c) 2003-2004 Benedikt Meurer\n",
+      "copyright", "Copyright \302\251 2003-2024 The Xfce development team",
       "authors", auth, NULL);
-
-   if(icon)
-      g_object_unref(G_OBJECT(icon));
 }
 
 static void
